	title "Rozsvcen a zhasnn LED"

; Seril "Mikrokontrolry PIC", dl 32
; PE 08/2010
;
; Vt pringl
;

     LIST		p=16F88				; nastaven typu procesoru
     #INCLUDE	<P16F88.INC>		; definice promnnch pro tento procesor

; === NASTAVEN KONFIGURANHO SLOVA ===

	__CONFIG	_CONFIG1, _CP_OFF & _CCP1_RB0 & _DEBUG_OFF & _WRT_PROTECT_OFF & _CPD_OFF & _LVP_OFF & _BODEN_OFF & _MCLR_ON & _PWRTE_ON & _WDT_OFF & _INTRC_IO
	__CONFIG	_CONFIG2, _IESO_OFF & _FCMEN_OFF

; === DEFINICE PROMNNCH ===
	#DEFINE	light_up  status_bits, 1

LEDMEM	EQU	0x20		; potek bloku datov pamti s histori vstupnch stav

	CBLOCK	0x28
		delay			; prodleva ekac smyky
		steps			; poet krok vnitnho cyklu
		steps2			; poet sub-krok pro posun LED
		status_bits		; registr s rznmi stavovmi bity
		d1				; pomocn promnn pro ekac smyku
	ENDC

; === KONSTATNTY ===
DEF_STEPS	EQU	0x70	; poet krok, po kterch dojde k inkrementaci FSR
DEF_STEPS2	EQU	0x09	; poet sub-krok, po kterch dojde k posunu LED

; === PROGRAM ===

RESET	ORG		0x0000		; zatek programu na adrese 0000h

; Inicializace port
		BCF		STATUS, RP0	; vbr banky 0
		CLRF    PORTA		; inicializace PORTA
		CLRF    PORTB		; inicializace PORTB

		BSF		STATUS, RP0	; vbr banky 1
		CLRF	ANSEL		; piny AN<6:0> jako digitln I/O
		CLRF	TRISA		; Piny RA<7:0> jako vstupy
		CLRF	TRISB		; Piny RB<7:0> jako vstupy

; Inicializace internho osciltoru
		MOVLW	b'01110000'
		MOVWF	OSCCON		; nastaven kmitotu internho osciltoru na 8 MHz

; Inicializace promnnch
		BCF		STATUS, RP0	; vbr banky 0
		MOVLW	b'00000010'
		MOVWF	status_bits ; light_up = 1, program zan rozsvcenm LED
		MOVLW	DEF_STEPS	
		MOVWF	steps		; steps = DEF_STEPS
		MOVLW	DEF_STEPS2	
		MOVWF	steps2		; steps2 = DEF_STEPS2
		MOVLW	0xFF
		MOVWF	delay		; delay = FFh

		; inicializace FSR
		MOVLW	LEDMEM		; LEDMEM (20h) -> W -> FSR
		MOVWF	FSR			; FSR nyn ukazuje na prvn registr datovho bloku LEDMEM
		
		CLRF	INDF		; 0x00 --> INDF (LEDMEM 20h)

; Hlavn smyka programu
LOOP_M
		BTFSC	light_up	; light_up = 0?
		CALL	INC_FSR		; - ne, tj. light_up = 1 = md rozsvcen --> inkrementace FSR

		MOVF	INDF, W		; INDF -> W
		MOVWF	PORTB		; W -> PORTB (tj. na vstupn piny portu B)
		CALL	DELAY0		; voln ekac smyky
		CLRF	PORTB		

		BTFSS	light_up	; light_up = 1?
		CALL	DEC_FSR		; - ne, tj. light_up = 0 = md zhasnn --> dekrementace FSR
		
		BCF		STATUS, C	
		RRF		delay, f	; plat piblin delay = delay / 2 (sekvence 255, 127, 63, 31, 15, 7, 3, 1)
		BTFSC	delay, 0	; je delay = 0?
		GOTO	LOOP_M		;  - ne ... skok na LOOP_M
							;  - ano ... znamen, e jsme ji proli vech 8 minulch stav

		DECF	delay, f	; delay: 00h -> FFh (255)
		DECFSZ	steps, f	; steps = steps - 1; je steps = 0?
		GOTO	LOOP_M		;  - ne: skok na LOOP_M,
							;  - ano: pokrauj dle
		MOVLW	DEF_STEPS	
		MOVWF	steps		; steps = DEF_STEPS

		MOVF	INDF, W		; star vstup -> W
		CALL	INC_FSR		; inkrementace FSR ukazatele
		MOVWF	INDF		; W -> nov vstup, kter se v nsledujcm podprogramu aktualizuje

		CALL	CHANGE_LED	; zmna vstupu
		GOTO	LOOP_M


; Podprogram pro zmnu vstupu
CHANGE_LED
		DECFSZ	steps2, f	; steps2 = steps2 - 1, vsledek = 0?
		RETURN				;  - ne: nvrat z podprogramu
		MOVLW	DEF_STEPS2	;  - ano: prove posun LED...
		MOVWF	steps2		; steps2 = DEF_STEPS2
		
		BTFSC	light_up	; light_up = 0?
		GOTO	CHANGE_LED2	;  - ne: jedn se o md rozsvcen, skok na CHANGE_LED2
							;  - ano: jedn se o md zhasnn ...
		BCF		STATUS, C	; C = 0
		RLF		INDF, f		; rotace vlevo
		BTFSS	STATUS, C	; C = 1?
		BSF		light_up	;  - ne: nastav md rozsvcen
		RETURN				;  - ano: nvrat

CHANGE_LED2					; md rozsvcen
		BSF		STATUS, C	; C = 1
		RLF		INDF, f		; rotace vlevo
		BTFSC	STATUS, C	; C = 0?
		BCF		light_up	;  - ne: nastav md zhasnn
		RETURN				;  - ano: nvrat

; Podprogram pro inkrementaci FSR
INC_FSR
		INCF	FSR, f		; FSR = FSR + 1
		BTFSC	FSR, 3		; FSR = 28h?
		BCF		FSR, 3		;  - ano: FSR = 20h
		RETURN

; Podprogram pro dekrementaci FSR
DEC_FSR
		DECF	FSR, f		; FSR = FSR - 1
		MOVF	FSR, W		; FSR -> W
		SUBLW	LEDMEM - 1	; 19h - W
		BTFSS	STATUS, Z	; vsledek = 0?
		RETURN				;  - ne: konec
		MOVLW	LEDMEM + 7	;  - ano: 27h -> W
		MOVWF	FSR			;  W -> FSR
		RETURN

; Podprogram s ekac smykou
DELAY0
		MOVF	delay, W
		MOVWF	d1
DELAY1	DECFSZ	d1, f		; d1 = d1 - 1
		GOTO	DELAY1		; je-li d1 > 0, provede se skok na DELAY1
		RETURN				; je-li d1 = 0, provede se nvrat z podprogramu

		END