	title "Odstrann zkmit tlatka dvojm tenm s prodlevou"

; Seril "Mikrokontrolry PIC", dl 33
; PE 09/2010
;
; Vt pringl
;
; Popis: Program pomoc dvou tlatek pipojench na vstupy RA0 a RA1
; reguluje v osmi krocch rychlost bcho svtla.

     LIST		p=16F88				; nastaven typu procesoru
     #INCLUDE	<P16F88.INC>		; definice promnnch pro tento procesor

; === NASTAVEN KONFIGURANHO SLOVA ===
	__CONFIG	_CONFIG1, _CP_OFF & _CCP1_RB0 & _DEBUG_OFF & _WRT_PROTECT_OFF & _CPD_OFF & _LVP_OFF & _BODEN_OFF & _MCLR_ON & _PWRTE_ON & _WDT_OFF & _INTRC_IO
	__CONFIG	_CONFIG2, _IESO_OFF & _FCMEN_OFF

; === DEFINICE PROMNNCH ===
	#DEFINE	rot_right	status_bits, 0		; bit, kter uruje smr rotace

	CBLOCK	0x20
		in				; hodnoty vstup tlatek
		status_bits		; registr se stavovmi bity
		d1				; pomocn promnn pro ekac smyku
		count_set		; poet cykl regulovateln pomoc tlatek
		count			; aktuln poet cykl pro posun LED
	ENDC

; === KONSTATNTY ===
DEF_COUNT	EQU	b'00001000'	; poten poet cykl pro count_set
DL			EQU	0x30		; vchoz poet cykl pro ekac smyku
MASK		EQU	b'00000011'	; zamaskovn nepouitch bit portu A; na pozicch 
							; odpovdajcch pipojenm vstupnm tlatkm je 1

; === PROGRAM ===
RESET	ORG		0x0000		; zatek programu na adrese 0000h

; Inicializace port
		BCF		STATUS, RP0	; vbr banky 0
		CLRF	PORTA		; inicializace PORTA
		MOVLW	0x01
		MOVWF	PORTB		; inicializace PORTB

		BSF		STATUS, RP0	; vbr banky 1
		CLRF	ANSEL		; piny AN<6:0> jako digitln I/O
		MOVLW	b'00000011'
		MOVWF	TRISA		; Piny RA<7:2> jako vstupy, RA0 a RA1 jako vstupy
		CLRF	TRISB		; Piny RB<7:0> jako vstupy

; Inicializace promnnch
		BCF		STATUS, RP0	; vbr banky 0
		CLRF	in
		CLRF	status_bits
		MOVLW	DEF_COUNT
		MOVWF	count_set	; count_set = DEF_COUNT
		MOVWF	count		; count = DEF_COUNT

; Hlavn smyka programu
LOOP_M
		MOVF	PORTA, W	; PORTA -> W
		ANDLW	MASK		; zamaskovn nepouitch bit
		MOVWF	in			; stav vstup RA0 a RA1 -> in
		CALL	CHANGE_LED	; aktualizace vstupu a ekac smyka
		MOVF	PORTA, W	; PORTA -> W
		ANDLW	MASK		; zamaskovn nepouitch bit
		SUBWF	in, W		; porovnn novho vstupu a vstupu v in
		BTFSS	STATUS, Z	; jsou stejn? (je vsledek porovnn 0, tj. Z=1?)
		GOTO	LOOP_M		;  - ne: neplatn vstup
		COMF	in, W		;  - ano: invertuj in -> W
		ANDLW	MASK		; zamaskovn nepouitch bit
		BTFSC	STATUS, Z	; je vsledek nenulov?
		GOTO	LOOP_M		; je-li vsledek nulov, nebylo stisknuto dn tlatko
LOOP1	CALL	CHANGE_LED	; aktualizace vstupu a ekac smyka
		MOVF	PORTA, W	; PORTA -> W
		ANDLW	MASK		; zamaskovn nepouitch bit
		XORLW	MASK		; invertovn bit 0 a 1
		BTFSS	STATUS, Z	; je vsledek nulov?
		GOTO	LOOP1		;  - ne: njak tlatko je stle sepnuto -> ekej
		CALL	HANDLE_BUTTONS	; - ano: obsluha tlatek
		GOTO	LOOP_M

; Podprogram pro rotaci LED
CHANGE_LED
		CALL	DELAY
		DECFSZ	count, f	; count = count - 1, vsledek = 0?
		RETURN				;  - ne: nvrat
		MOVF	count_set, W	;  - ano: count = count_set a pokrauj rotac
		MOVWF	count

		BCF		STATUS, C	; tento bit se pouije pi nsledujc rotaci
		BTFSC	rot_right	; rot_right = 0?
		GOTO	CHANGE_ROT_RIGHT	; - ne: rotuj vpravo
									; - ano: rotuj vlevo
		RLF		PORTB, f	; rotace vlevo
		BTFSC	PORTB, 7	; bit7 = 0?
		BSF		rot_right	;  - ne: zmna pznaku rot_right = 1
		RETURN				;  - ano: konec

CHANGE_ROT_RIGHT
		RRF		PORTB, f	; rotace vpravo
		BTFSC	PORTB, 0	; bit0 = 0?
		BCF		rot_right	;  - ne: zmna pznaku rot_right = 0
		RETURN				;  - ano: konec

; Podprogram definujc akce pro jednotliv tlatka
HANDLE_BUTTONS
		BTFSS	in, 0		; RA0 = 1?
		GOTO	DEC_SPEED	;  - ne: tlatko na RA0 bylo sepnuto, zmeni rychlost
		BTFSS	in, 1		; RA1 = 1?
		GOTO	INC_SPEED	;  - ne: tlatko na RA1 bylo sepnuto, zvti rychlost
		RETURN				; dn tlatko nebylo sepnuto ... konec

INC_SPEED					; zvten rychlosti bcho svtla
		BCF		STATUS, C
		BTFSS	count_set, 0
		RRF		count_set, f
		RETURN

DEC_SPEED					; zmnen rychlosti bcho svtla
		BCF		STATUS, C
		BTFSS	count_set, 7
		RLF		count_set, f
		RETURN

; Podprogram s ekac smykou
DELAY
		MOVLW	DL			; d1 = DL
		MOVWF	d1
DELAY1	DECFSZ	d1, f		; d1 = d1 - 1
		GOTO	DELAY1		; je-li d1 > 0, provede se skok na DELAY1
		RETURN				; je-li d1 = 0, provede se nvrat z podprogramu

		END