	title "Kdov zmek"

; Seril "Mikrokontrolry PIC", dl 36
; PE 12/2010
;
; Vt pringl
;
; Popis: Kdov zmek s dvoubarevnou LED indikujc aktuln stav.

     LIST		p=16F88				; nastaven typu procesoru
     #INCLUDE	<P16F88.INC>		; definice promnnch pro tento procesor

; === NASTAVEN KONFIGURANHO SLOVA ===
	__CONFIG	_CONFIG1, _CP_OFF & _CCP1_RB0 & _DEBUG_OFF & _WRT_PROTECT_OFF & _CPD_OFF & _LVP_OFF & _BODEN_OFF & _MCLR_ON & _PWRTE_ON & _WDT_OFF & _INTRC_IO
	__CONFIG	_CONFIG2, _IESO_OFF & _FCMEN_OFF

; === DEFINICE PROMNNCH ===
	CBLOCK	0x20
		keycode			; kd klvesy
		row_in			; naten dek z klvesnice			
		colcount		; potadlo sloupc
		cur_key			; ukazatel na souasnou pozici v sekvenci uloench kd stisknutch klves v pamti
		keynum			; seln kd klvesy
		green_led		; sekvence pro zelenou LED
		red_led			; sekvence pro ervenou LED
		led_mask		; maska pro vbr bitu z registr green_led/red_led
		d1, d2			; pomocn promnn pro ekac smyku
		tout1, tout2	; pomocn promnn pro vpoet prodlevy pro ukonen reimu zadvn kdu
	ENDC

W_temp		EQU	0x70	; doasn uloen obsahu registru W pi vyvoln peruen
status_temp	EQU	0x71	; doasn uloen obsahu registru STATUS pi vyvoln peruen
	
; === KONSTATNTY ===
DEF_TIMEOUT		EQU	0x10		; nastaven doby, za kterou se automaticky deaktivuje md zadvn kdu
KEY_MEM_START	EQU	0x40		; zatek pamti, kam se m ukldat sekvence kd stisknutch klves
KEY_MEM_END		EQU	0x50		; konec pamti s kdy klves
GLED_ENTER		EQU	b'00001111'	; sekvence pro zel. LED pi zadvn kdu
RLED_ENTER		EQU	b'11110000'	; sekvence pro erv. LED pi zadvn kdu
GLED_CONFIRM	EQU	b'10101010'	; sekvence pro zel. LED pi sprvnm zadn kdu
RLED_BLOCK		EQU	b'10101010'	; sekvence pro erv. LED pi nesprvnm zadn kdu
RLED_PASSIVE	EQU	b'11111111'	; sekvence pro erv. LED v mdu neinnosti

; === DEFINICE BIT ===
#DEFINE		GLED_PIN	PORTA, 0	; pin, na kter je pipojena zel. LED
#DEFINE		RLED_PIN	PORTA, 1	; pin, na kter je pipojena erv. LED
#DEFINE		RELAY_PIN	PORTA, 2	; pin, na kter je pipojeno rel

; === VEKTOR RESETU ===
RESET	ORG		0x0000		; zatek programu na adrese 0000h
	GOTO	INIT

; === OBSLUHA PERUEN ===
; Peruen vyvolan peteenm asovae TMR0 ovld pipojenou dvoubarevnou indikan LED
ISR		ORG		0x0004		; vektor peruen
; Uloen obsahu registr W a STATUS
	MOVWF	W_temp
	MOVF	STATUS,W		; W -> W_temp
	MOVWF	status_temp		; STATUS -> status_temp
	
	BCF		STATUS, RP0		; vbr banky 0
	; nastaven rovn na pinu, na kter je pipojena zelen LED
	MOVF	led_mask, W
	ANDWF	green_led, W	; led_mask vybere jeden bit z registru green_led
	BTFSS	STATUS, Z		; a podle nj se nastav rove na pslunm I/O pinu
	BSF		GLED_PIN
	BTFSC	STATUS, Z
	BCF		GLED_PIN

	; nastaven rovn na pinu, na kter je pipojena erven LED
	MOVF	led_mask, W
	ANDWF	red_led, W		; led_mask vybere jeden bit z registru red_led
	BTFSS	STATUS, Z		; a podle nj se nastav rove na pslunm I/O pinu
	BSF		RLED_PIN
	BTFSC	STATUS, Z
	BCF		RLED_PIN

	; rotace masky
	RRF		led_mask, f		; rotace vpravo
	BCF		led_mask, 7
	BTFSC	STATUS, C		; bit0 -> bit7
	BSF		led_mask, 7

	; vynulovn pznaku peruen od asovae TMR0
	BCF		INTCON, 2
	
; Nahrn obsahu registr W a STATUS
	MOVF    status_temp,W
	MOVWF   STATUS
	SWAPF   W_temp,f
	SWAPF   W_temp,W
	RETFIE					; nvrat z peruen

; === PROGRAM ===
INIT
; Inicializace port
	BCF		STATUS, RP0		; vbr banky 0
	CLRF	PORTA			; inicializace PORTA
	CLRF	PORTB			; inicializace PORTB

	BSF		STATUS, RP0		; vbr banky 1
	CLRF	ANSEL			; piny AN<6:0> jako digitln I/O
	CLRF	TRISA			; Piny RA<7:0> jako vstupy
	MOVLW	0xFF
	MOVWF	TRISB			; Piny RB<7:0> jako vstupy

; Inicializace asovae a pull-up rezistor (registr OPTION_REG)
	MOVLW	b'01010001'
;			  ||||||||
;			  |||||*** ----- nastaven dlcho pomru peddliky (1:4)
;			  ||||* -------- piazen peddliky (asova TMR0)
;			  |||* --------- nastaven hrany pro inkrementaci TMR0 pi ext. zdroji signlu (spdov hrana)
;			  ||* ---------- vbr zdroje signlu pro TMR0 (intern instrukn cyklus)
;			  |* ----------- nastaven hrany pro vyvoln peruen na pinu RB0/INT (nb.hr.)
;			  * ------------ pull-up rezistory na PORTB (povoleno)
	MOVWF	OPTION_REG

; Inicializace internho osciltoru
	;MOVLW	b'00100000'
	;MOVWF	OSCCON			; nastaven kmitotu internho osciltoru na 250 kHz

; Nastaven peruen
	MOVLW	b'10100000'
	MOVWF	INTCON			; povoleno peruen pouze od asovae TMR0

	BCF		STATUS, RP0		; vbr banky 0

; Inicializace promnnch
	MOVLW	0x01
	MOVWF	led_mask
	CLRF	d1
	CLRF	d2
	
	CALL	RESET_TIMEOUT
	CALL	RESET_SEQ

	CALL	LOAD_CODE		; nahraje do pamti pstupov kd

; Hlavn smyka programu
MAIN
	CALL	TIMEOUT		; odpotv dobu, po kter se deaktivuje md zadvn kdu
	CALL	KEYSCAN		; ten klvesnice, kd klvesy se ulo do W
	MOVWF	keynum		; keynum = W (kde se nachz kd klvesy)
	SUBLW	0x10		; byla stisknuta njak klvesa? (kd 10h zna, e nebyla dn klvesa stisknuta)
	BTFSC	STATUS, Z
	GOTO	MAIN
MAIN1					; bylo-li njak tlatko sepnuto, pokej na jeho rozepnut
	CALL	RESET_TIMEOUT	; po stisknut jakkoliv klvesy se nastav max. as pro timeout
	CALL	KEYSCAN		; ten klvesnice
	SUBLW	0x10
	BTFSS	STATUS, Z	; je stle stisknuta njak klvesa?
	GOTO	MAIN1		;  - ano (Z=0): skok na MAIN1
; Test, zda byla stisknuta numerick klvesa (0-9)
	MOVF	keynum, W
	SUBLW	0x09		; 9 - keynum
	BTFSS	STATUS, C	; C = 1?  (= nedolo k penosu...keynum <= 9)
	GOTO	MAIN2		;  - ne: skok na MAIN2
						;  - ano: byla stisknuta num. klvesa...
	MOVLW	GLED_ENTER	; nastav pslun sekvence na erven a zelen LED ...
	MOVWF	green_led	;  ... indikujc md zadvn kdu
	MOVLW	RLED_ENTER
	MOVWF	red_led
	
	MOVF	cur_key, W
	MOVWF	FSR			; FSR = cur_key
	MOVF	keynum, W	
	MOVWF	INDF		; keynum se ulo do registru na adrese cur_key
	
	INCF	cur_key, f	; cur_key = cur_key + 1 (inkrementace adresy)
	MOVF	cur_key, W	; cur_key -> W
	SUBLW	KEY_MEM_END	; test, zda-li cur_key neodkazuje na adresu konce pamovho bloku...
	BTFSC	STATUS, Z	;  ... vylennho pro naten vstupnho kdu. Pokud ano, ...
	CALL	RESET_SEQ	;  ... program pejde do vchozho stavu.
	GOTO	MAIN
	
; Test, zda byla stisknuta klvesa ENTER (tj. keycode = Ah)
MAIN2
	MOVF	keynum, W
	SUBLW	0x0A
	BTFSS	STATUS, Z	; Ah - keynum = 0?
	GOTO	MAIN3		;  - ne: skok na MAIN3
						;  - ano: byla stisknuta klvesa ENTER
	CALL	CHECK_CODE	; ov zadan kd
	GOTO	MAIN
	
; Test, zda nebyla stiknuta klvesa CANCEL (tj. keycode = Bh)
MAIN3
	MOVF	keynum, W
	SUBLW	0x0B
	BTFSS	STATUS, Z	; Bh - keynum = 0?
	GOTO	MAIN4		;  - ne: skok na MAIN4
						;  - ano: byla stisknuta klvesa CANCEL
	CALL	RESET_SEQ	; zru md zadvn kdu
	GOTO	MAIN
	
MAIN4
	; dal klvesy...
	GOTO	MAIN

; Podprogram, kter zru md zadvn kdu a uvede program do vchozho stavu,
; ve kterm se ek na stisk prvn klvesy
RESET_SEQ
	MOVLW	KEY_MEM_START
	MOVWF	cur_key
	CLRF	green_led
	MOVLW	RLED_PASSIVE
	MOVWF	red_led
	BCF		RELAY_PIN
	RETURN
	
; Ov sprvnost zadanho kdu porovnnm s kdem, kter byl nahrn do pamti pi
; inicializaci programu. Oba kdy jsou uloeny na stejnch adresch, ale v jinch bankch.
; (tj. adresy se li v bitu 7)
CHECK_CODE
	MOVF	cur_key, W
	MOVWF	FSR			; FSR = cur_key (ukazuje na adresu za posledn zadanou slic)
	BSF		FSR, 7		; banka 1
	MOVF	INDF, W
	SUBLW	0xF0		; kd uloen za posledn platnou slic, pokud cur_key neodkazuje na ...
	BTFSS	STATUS, Z	;  ... adresu, kde je uloen tento kd, nebyl zadn sprvn poet slic ...
	GOTO	CHECK_WRONG	;  ... a kd se automaticky vyhodnot jako neplatn
	BCF		FSR, 7		; banka 0
	DECF	FSR, f		; FSR = FSR - 1 (odkazuje na posledn zadanou slici)

CHECK_LOOP
	MOVF	INDF, W
	BSF		FSR, 7		; banka 1
	SUBWF	INDF, W		; porovnn pslunch slic kdu
	BTFSS	STATUS, Z
	GOTO	CHECK_WRONG	; nejsou-li stejn, kd je neplatn
	
	; oven, zda ji nebyla porovnna vechna odpovdajc pamov msta
	BCF		FSR, 7		; banka 0
	MOVF	FSR, W
	SUBLW	KEY_MEM_START	; W = KEY_MEM_START - FSR (porovnn adres)
	BTFSC	STATUS, Z		; vsledek = 0?
	GOTO	CHECK_CORRECT	;  - ano (tj.Z=1): ji byly oveny vechny slice a kd je sprvn
	DECF	FSR, f			;  - ne (tj.Z=0): FSR = FSR - 1 (ov dal slici)
	GOTO	CHECK_LOOP
	
; nesprvn kd
CHECK_WRONG
	CLRF	green_led	; nastav odpovdajc sekvence na erv. a zel. LED
	MOVLW	RLED_BLOCK
	MOVWF	red_led
	CALL	DELAY		; blokujc prodleva
	CALL	RESET_SEQ	; nastav vchoz stav
	RETURN

; sprvn kd
CHECK_CORRECT
	MOVLW	GLED_CONFIRM
	MOVWF	green_led	; nastav odpovdajc sekvence na erv. a zel. LED
	CLRF	red_led
	BSF		RELAY_PIN	; aktivace rel
	CALL	DELAY		; prodleva, po kterou je rel aktivovno	
	CALL	RESET_SEQ	; nastav vchoz stav
	RETURN
	
; Podprogram pro scan klvesnice
KEYSCAN
	CLRF	keycode		; keycode = 0
	BSF		STATUS, RP0	; banka 1
	MOVLW	0xEF
	MOVWF	TRISB		; RB4 jako vstup
	BCF		STATUS, RP0	; banka 0
ROWSCAN
	CLRF	PORTB
	MOVF	PORTB, W	; ten aktivovanho dku klvesnice
	MOVWF	row_in		; row_in = PORTB
	MOVLW	.4
	MOVWF	colcount	; colcount = 4
COLSCAN
	RRF		row_in, f
	BTFSS	STATUS, C	; jestlie C = 0, byla stisknuta klvesa
	GOTO	KEY_TABLE	; dekduj klvesu a nvrat
	INCF	keycode, f	; keycode = keycode + 1
	DECFSZ	colcount, f	; colcount = colcount - 1, vsledek = 0?
	GOTO	COLSCAN		;  - ne: vechny sloupce jet nebyly peteny
						;  - ano: aktivuj dal dek
	BSF		STATUS, C	; C = 1
	BSF		STATUS, RP0	; banka 1
	RLF		TRISB, f	; dal dek
	BCF		STATUS, RP0 ; banka 0
	BTFSC	STATUS, C	; C = 0?
	GOTO	ROWSCAN		;  - ne: jet nebyly peteny vechny dky
	RETLW	0x10		; dn klvesa nebyla stisknuta, nvrat (W = 10h)

; Datov tabulka s kdy klves
KEY_TABLE
	MOVF	keycode, W
	ADDWF	PCL, f
	DT 	0x01,0x02,0x03,0x0A,0x04,0x05,0x06,0x0B
	DT	0x07,0x08,0x09,0x0C,0x0E,0x00,0x0F,0x0D

; asov prodleva
DELAY
	DECFSZ	d1, f
	GOTO	DELAY
	DECFSZ	d2, f
	GOTO	DELAY
	RETURN

; Nahrn kdu do pamti RAM
LOAD_CODE
	MOVLW	KEY_MEM_START
	MOVWF	FSR			; FSR = KEY_MEM_START
LOAD_CODE1
	CALL	CODE_TABLE
	BSF		FSR, 7		; banka 1
	MOVWF	INDF
	SUBLW	0xF0		; konec kdov sekvence?
	BTFSC	STATUS, Z
	RETURN
	INCF	FSR, f		; FSR = FSR + 1
	GOTO	LOAD_CODE1

; Tabulka s pstupovm kdem
CODE_TABLE
	BCF		FSR, 7
	MOVLW	KEY_MEM_START
	SUBWF	FSR, W		; W = FSR - KEY_MEM_START
	ADDWF	PCL, f
	DT	0x01, 0x02, 0x03, 0x04, 0xF0

; Dekrementace asomry, kter uruje dobu, po kter se zru md zadvn kdu
; a program pejde do vchozho stavu (odpovd stisknut klvesy CANCEL)
TIMEOUT
	DECFSZ	tout1, f
	RETURN
	DECFSZ	tout2, f
	RETURN
	CALL	RESET_SEQ
	RETURN

; Reset asomry
RESET_TIMEOUT
	CLRF	tout1
	MOVLW	DEF_TIMEOUT
	MOVWF	tout2
	RETURN

	END