	title "Obsluha klvesnice se zobrazenm na LED displeji a uloenm do pamti EEPROM"

; Seril "Mikrokontrolry PIC", dl 39
; PE 03/2011
;
; Vt pringl
;
; Popis: Program zobraz na sedmisegmentovm LED displeji 
; aktuln stisknutou klvesu na 4x4 maticov klvesnici.
; Po stisknut klvesy "#" se ulo sekvence stisknutch 
; klves do pamti EEPROM. Po resetu mikrokontrolru se 
; sekvence sel uloench v pamti EEPROM zobraz na displeji.

     LIST		p=16F88				; nastaven typu procesoru
     #INCLUDE	<P16F88.INC>		; definice promnnch pro tento procesor

; === NASTAVEN KONFIGURANHO SLOVA ===
	__CONFIG	_CONFIG1, _CP_OFF & _CCP1_RB0 & _DEBUG_OFF & _WRT_PROTECT_OFF & _CPD_OFF & _LVP_OFF & _BODEN_OFF & _MCLR_ON & _PWRTE_ON & _WDT_OFF & _INTRC_IO
	__CONFIG	_CONFIG2, _IESO_OFF & _FCMEN_OFF

; === DEFINICE PROMNNCH ===
	CBLOCK	0x20
		keycode			; kd klvesy
		row_in			; naten dek z klvesnice			
		colcount		; potadlo sloupc
		cur_key			; ukazatel na souasnou pozici v sekvenci uloench kd stisknutch klves v pamti
		keynum			; seln kd klvesy
		cur_eaddr		; ukazatel na adresu pamti EEPROM
		d1, d2			; pomocn promnn pro ekac smyku
	ENDC

; === KONSTATNTY ===
KEY_MEM_START	EQU	0x40	; zatek pamti, kam se m ukldat sekvence kd stisknutch klves
KEY_MEM_END		EQU	0x50	; konec pamti s kdy klves

; === EEPROM ===
	ORG	0x2100
	DE	0,1,2,3,4,5,6,7,8,9,0x0A,0x0B,0x0C,0x0D,0x0E,0xF0

; === PROGRAM ===
RESET	ORG		0x0000		; zatek programu na adrese 0000h

; Inicializace port
	MOVLW	0xFF
	MOVWF	PORTA		; inicializace PORTA
	CLRF	PORTB		; inicializace PORTB

	BSF		STATUS, RP0	; vbr banky 1
	CLRF	ANSEL		; piny AN<6:0> jako digitln I/O
	CLRF	TRISA		; piny RA<7:0> jako vstupy
	MOVLW	0xFF
	MOVWF	TRISB		; piny RB<7:0> jako vstupy
	BCF		OPTION_REG, 7	; Intern pull-up rezistory na portu B aktivn

	BCF		STATUS, RP0	; vbr banky 0

; Inicializace promnnch
	CALL	RESET_MEM_POINTER

; Zobraz sekvenci kd uloench v pamti EEPROM na displeji
	CALL	READ_EEPROM
	CALL	DISPLAY_CODES

; Hlavn smyka programu
MAIN
	CALL	KEYSCAN		; ten klvesnice, kd klvesy se ulo do W
	MOVWF	keynum		; keynum = W (kde se nachz kd klvesy)
	CALL	DISP_TABLE	; dekdovan segment pro zobrazen na LED displeji ( -> W)
	MOVWF	PORTA		; zobrazen na displeji
	MOVF	keynum, W
	SUBLW	0x10		; byla stisknuta njak klvesa? (kd 10h zna, e nebyla dn klvesa stisknuta)
	BTFSC	STATUS, Z
	GOTO	MAIN
MAIN1					; bylo-li njak tlatko sepnuto, pokej na jeho rozepnut
	CALL	KEYSCAN		; ten klvesnice
	SUBLW	0x10
	BTFSS	STATUS, Z	; je stle stisknuta njak klvesa?
	GOTO	MAIN1		;  - ano (Z=0): skok na MAIN1

; Test, zda byla stisknuta numerick klvesa # reprezentovan kdem Fh
	MOVF	keynum, W
	SUBLW	0x0F
	BTFSC	STATUS, Z	; Fh - keynum = 0?
	GOTO	MAIN2		; - ano: skok na MAIN2
						; - ne: nebyla stisknuta klvesa #, ulo kd do vyrovnvac pamti
	MOVF	cur_key, W
	MOVWF	FSR			; FSR = cur_key
	MOVF	keynum, W	
	MOVWF	INDF		; keynum se ulo do registru na adrese cur_key
	
	INCF	cur_key, f	; cur_key = cur_key + 1 (inkrementace adresy)
	MOVF	cur_key, W	; W = cur_key
	SUBLW	KEY_MEM_END	; Test, zda-li cur_key neodkazuje na adresu konce pamovho bloku...
	BTFSC	STATUS, Z	;  ... vylennho pro naten vstupnho kdu.
	CALL	RESET_MEM_POINTER	; - pokud ano, nastav ukazatel na zatek pamti
	GOTO	MAIN

; Byla stisknuta klvesa #: ulo naten kd do pamti EEPROM
MAIN2
	MOVF	cur_key, W
	MOVWF	FSR			; FSR = cur_key
	MOVLW	0xF0
	MOVWF	INDF		; do registru na adrese cur_key se ulo kd F0h indikujc konec sekvence kd
	CALL	WRITE_EEPROM
	CALL	RESET_MEM_POINTER
	GOTO	MAIN

; Podprogram pro nastaven vchoz adresy pro cur_key ukazatel
RESET_MEM_POINTER
	MOVLW	KEY_MEM_START
	MOVWF	cur_key
	RETURN

; Podprogram pro scan klvesnice
KEYSCAN
	CLRF	keycode		; keycode = 0
	BSF		STATUS, RP0	; banka 1
	MOVLW	0xEF
	MOVWF	TRISB		; RB4 jako vstup
	BCF		STATUS, RP0	; banka 0
ROWSCAN
	CLRF	PORTB
	MOVF	PORTB, W	; ten aktivovanho dku klvesnice
	MOVWF	row_in		; row_in = PORTB
	MOVLW	.4
	MOVWF	colcount	; colcount = 4
COLSCAN
	RRF		row_in, f
	BTFSS	STATUS, C	; jestlie C = 0, byla stisknuta klvesa
	GOTO	KEY_TABLE	; dekduj klvesu a nvrat
	INCF	keycode, f	; keycode = keycode + 1
	DECFSZ	colcount, f	; colcount = colcount - 1, vsledek = 0?
	GOTO	COLSCAN		;  - ne: vechny sloupce jet nebyly peteny
						;  - ano: aktivuj dal dek
	BSF		STATUS, C	; C = 1
	BSF		STATUS, RP0	; banka 1
	RLF		TRISB, f	; dal dek
	BCF		STATUS, RP0 ; banka 0
	BTFSC	STATUS, C	; C = 0?
	GOTO	ROWSCAN		;  - ne: jet nebyly peteny vechny dky
	RETLW	0x10		; dn klvesa nebyla stisknuta, nvrat (W = 10h)

; Datov tabulka s kdy klves
KEY_TABLE
	MOVF	keycode, W
	ADDWF	PCL, f
	DT 	0x01,0x02,0x03,0x0A,0x04,0x05,0x06,0x0B
	DT	0x07,0x08,0x09,0x0C,0x0E,0x00,0x0F,0x0D

; Datov tabulka dekdujc segmenty pro zobrazen na LED displeji
DISP_TABLE
	ADDWF	PCL, f
	DT	b'10000000',b'11011001',b'01000100',b'01010000'
	DT	b'00011001',b'00010010',b'00000010',b'11011000'
	DT	b'00000000',b'00010000',b'00001000',b'00000011'
	DT	b'10000110',b'01000001',b'00000110',b'00001110'
	DT	b'01011111'

; Podprogram pro uloen sekvence natench kd do pamti EEPROM
WRITE_EEPROM
	MOVLW	KEY_MEM_START
	MOVWF	FSR
	CLRF	cur_eaddr
WRITE_LOOP
	BANKSEL	EECON1			; Vbr banky 3, ve kter je reg. EECON1
	BTFSC	EECON1, WR		; Test: byl dokonen pedchoz zpis?
	GOTO	$-1				;  - ne: ekn (skok na pedchoz instr.)
	BANKSEL	cur_eaddr		; Vbr banky 0
	MOVF	cur_eaddr, W	; W = cur_addr (naten adresy)
	BANKSEL	EEADR			; Vbr banky 2, ve kter je reg. EEADR
	MOVWF	EEADR			; Nahrn adresy, na kterou chceme data zapsat
	MOVF	INDF, W			; W = indf (naten dat s vyuitm nepmho adresovn)
	MOVWF	EEDATA			; Nahrn dat, kter chceme uloit do pamti
	BANKSEL	EECON1			; Vbr banky 3, ve kter je reg. EECON1
	BCF		EECON1, EEPGD	; Nastaven pstupu k datov pamti EEPROM
	BSF		EECON1, WREN	; Povolen zpisu do pamti
	MOVLW	0x55			; W = 55h
	MOVWF	EECON2			; EECON2 = W
	MOVLW	0xAA			; W = AAh
	MOVWF	EECON2			; EECON2 = W
	BSF		EECON1, WR		; Zahjen zpisu do pamti
	BCF		EECON1, WREN 	; Zakzn zpisu do pamti	

	BANKSEL	cur_eaddr		; Vbr banky 0
	INCF	cur_eaddr, f	; inkrementace cur_eaddr
	MOVF	INDF, W
	SUBLW	0xF0
	BTFSC	STATUS, Z		; konec kdov sekvence?
	RETURN					; - ano: konec zpisu
	INCF	FSR, f			; - ne: zapi dal bajt
	GOTO	WRITE_LOOP

; Podprogram pro naten kdu z pamti EEPROM
READ_EEPROM
	MOVLW	KEY_MEM_START
	MOVWF	FSR				; FSR = KEY_MEM_START
	CLRF	cur_eaddr
READ_LOOP
	BANKSEL	cur_eaddr		; Vbr banky 0
	MOVF	cur_eaddr, W	; W = cur_addr (naten adresy)
	BANKSEL	EEADR			; Vbr banky 2, ve kter je reg. EEADR
	MOVWF	EEADR			; Nahrn adresy, ze kter chceme st data
	BANKSEL	EECON1			; Vbr banky 3, ve kter je reg. EECON1
	BCF		EECON1, EEPGD	; Nastaven pstupu k datov pamti EEPROM
	BSF		EECON1, RD		; Zahjen ten z pamti
	BANKSEL	EEDATA			; Vbr banky 2, ve kter je reg. EEDATA
	MOVF	EEDATA, W		; W = EEDATA
	MOVWF	INDF			; zpis dat do pamti RAM
	BANKSEL	cur_eaddr		; Vbr banky 0
	INCF	cur_eaddr, f	; inkrementace cur_eaddr
	MOVF	INDF, W
	SUBLW	0xF0
	BTFSC	STATUS, Z		; konec kdov sekvence?
	RETURN					; - ano: konec ten
	MOVF	FSR, W
	SUBLW	KEY_MEM_END		; test pro ppad, e se v pamti EEPROM nenachz kd F0h
	BTFSC	STATUS, Z		; je ji zaplnna pam RAM?
	RETURN					; - ano: konec ten
	INCF	FSR, f			; - ne: nati dal bajt
	GOTO	READ_LOOP

; Podprogram pro zobrazen kd na displeji
DISPLAY_CODES
	MOVLW	KEY_MEM_START
	MOVWF	FSR				; FSR = KEY_MEM_START

DISPLAY_LOOP
	MOVF	INDF, W
	SUBLW	0xF0
	BTFSC	STATUS, Z		; konec kdov sekvence?
	RETURN
	MOVF	INDF, W
	SUBLW	0x0F			; 0Fh - W: test na neplatn data (W > Fh?)
	BTFSS	STATUS, C		; C = 1?  (= nedolo k penosu...EEDATA <= Fh)
	RETURN					; - ne: nvrat
	MOVF	INDF, W			; - ano: pokrauj
	CALL	DISP_TABLE		; dekdovan segment pro zobrazen na LED displeji ( -> W)
	MOVWF	PORTA			; zobrazen na displeji
	CALL	DELAY			; ekej cca 0,8s
	INCF	FSR, f			; inkrementace adresy
	MOVF	FSR, W
	SUBLW	KEY_MEM_END		; test pro ppad, e se v pamti nenachz kd F0h
	BTFSS	STATUS, Z		; konec pamti RAM?
	GOTO	DISPLAY_LOOP	; - ne: zobraz dal kd
	RETURN					; - ano: konec

; Podprogram ekac smyky (cca 0,8s)
DELAY
	MOVLW	0x08
	MOVWF	d2
DELAY1
	DECFSZ	d1, f
	GOTO	DELAY1
	DECFSZ	d2, f
	GOTO	DELAY1
	RETURN

	END