	title "Penos dat mezi dvma mikrokontrolry prostednictvm sbrnice I2C - obvod master"

; Seril "Mikrokontrolry PIC", dl 56
; PE-AR 08/2012
;
; Vt pringl
;
; Popis: rovn na pinech RB5 a RB7 se pen pes I2C do dalho mikrokontrolru.

	LIST		p=16F88				; nastaven typu procesoru
	#INCLUDE	<P16F88.INC>		; definice promnnch pro tento procesor

; === NASTAVEN KONFIGURANHO SLOVA ===
	__CONFIG	_CONFIG1, _CP_OFF & _CCP1_RB0 & _DEBUG_OFF & _WRT_PROTECT_OFF & _CPD_OFF & _LVP_OFF & _BODEN_OFF & _MCLR_ON & _PWRTE_ON & _WDT_OFF & _INTRC_IO
	__CONFIG	_CONFIG2, _IESO_OFF & _FCMEN_OFF

; === DEFINICE PROMNNCH ===
	CBLOCK	0x20
		dataOut			; vstupn data pro vysln pes I2C
		count			; potadlo bit
		buffer			; reg. pro vstupn a vstupn datov bit
		txBuf			; buffer pro vysln dat
	ENDC

; === DEFINICE BIT ===
RBPU	EQU	0x7			; pull-up rezistory na portu B
SDA		EQU 0x1			; pin SDA
SCL		EQU	0x4			; pin SCL
DI		EQU	0x7			; vstupn datov bit
DO		EQU	0x6			; vstupn datov bit

; === KONSTANTY ===
	#DEFINE	SLAVEADDR	b'10101110'		; adresa zazen I2C slave (posledn bit je RW bit = 0 -> vdy zpis)


; === VEKTOR RESETU ===
RESET	ORG		0x0000		; zatek programu na adrese 0000h
	
; Inicializace port
	CLRF	PORTA		; inicializace portu A
	CLRF	PORTB		; inicializace portu B

	BSF		STATUS, RP0	; vbr banky 1

	CLRF	ANSEL		; piny AN<6:0> jako digitln I/O
	CLRF	TRISA		; piny RA<7:0> jako vstupy
	MOVLW	b'11110010'	
	MOVWF	TRISB		; piny RB<7:5> a pin RB1 (SDA) a RB4 (SCL) jako vstupy, ostatn jako vstupy

	CLRF	SSPSTAT		; inicializace modulu SSP

	BSF		OSCCON, IRCF1		; Fosc = 250 kHz

	BCF		OPTION_REG, RBPU	; povolen pull-up rezistor na portu B

	BCF		STATUS, RP0	; vbr banky 0

	MOVLW	b'00111011'
	MOVWF	SSPCON		; povolen hod. signlu (CKP=1), md I2C softwarov master (slave deaktivovn), povolen SSP

MAIN
	BCF		STATUS, RP0	; vbr banky 0
	MOVF	PORTB, W	; PORTB -> W
	ANDLW	b'11100000'	; maskovn bit
	MOVWF	dataOut		; vstupn data
	CALL	WRBYTE		; odesln dat pes I2C

	GOTO	MAIN

; Podprogram pro penos dat ped I2C
; (vstup: data z reg. dataOut)
WRBYTE
	BCF		STATUS, RP0		; banka 0
	MOVLW	SLAVEADDR		; nastaven adresy zazen slave
	MOVWF	txBuf			; adresa do TX bufferu
	CALL	BSTART			; generovn podmnky START
	CALL	TX				; vysln adresy zazen slave
	BCF		STATUS, RP0		; banka 0
	MOVF	dataOut, W		; dataOut -> W
	MOVWF   txBuf			; W -> txBuf
	CALL	TX				; voln procedury TX pro vysln dat
	CALL	BSTOP			; generovn podmnky STOP
	RETURN

; Podprogram pro odesln 8 bit
TX
	BCF		STATUS, RP0		; banka 0
	MOVLW	.8
	MOVWF	count			; count = 8 (potadlo bit)

TXLP
	BCF		buffer, DO		; txBuf<7> -> DO
	BTFSC	txBuf, 7
	BSF		buffer, DO

	CALL	BITOUT			; sriov vstup dat
	RLF		txBuf, f		; rotace txBuf vlevo
	BTFSS	STATUS, C		; F(6) -> F(7)
	BCF		txBuf, 0		; F(7) -> C
	BTFSC	STATUS, C		; C -> F(0)
	BSF		txBuf, 0
	DECFSZ	count, f		; count = count - 1, vsledek 0?
	GOTO	TXLP			;  - ne: skok na TXLP
	CALL	BITIN			;  - ano: peti ACK bit
	BTFSC	buffer, DI		; zkontroluj ACK bit (ACK mus bt roven L)
	RETLW	.3				; chyba, penos nepotvrzen, nvrat z podprogramu
	BCF		STATUS, RP0		; banka 0
	RETLW	0				; nvrat z podprogramu (dn chyba)

; Procedura pro generovn podmnky START
; (SDA H->L zatmco SCL = H)
BSTART
	BSF     STATUS, RP0		; banka 1
	BSF		TRISB, SDA		; SDA = H (stav vysok impedance)
	BSF		TRISB, SCL		; SCL = H (stav vysok impedance)
	BCF		STATUS, RP0		; banka 0
	BTFSS	PORTB, SCL		; sbrnice je blokovna (SCL = L)?
	RETLW	.1				; chyba, nvrat z podprogramu
	BSF		STATUS, RP0		; banka 1
	BCF		TRISB, SDA		; SDA -> L (zatmco SCL = H)
	NOP						; 5us @2MHZ
	NOP
	NOP
	BCF		TRISB, SCL		; SCL -> L, zatek penosu dat
	BCF		STATUS, RP0		; banka 0
	RETLW	0

; Procedura pro generovn podmnky STOP
; (SDA L->H zatmco SCL = H)
BSTOP
	BSF		STATUS, RP0		; banka 1
	BCF		TRISB, SDA		; SDA = L
	BSF		TRISB, SCL		; SCL = H (stav vysok impedance)
	NOP
	NOP
	NOP
	BCF		STATUS, RP0		; banka 0
	BTFSS	PORTB, SCL		; je SCL = H?
	RETLW	.1				;  - ne: chyba, nvrat z podprogramu
	BSF		STATUS, RP0		; banka 1
	BSF		TRISB, SDA		; SDA -> H (zatmco SCL = H)
	BTFSS	TRISB, SDA		; je SDA = H?
	RETLW	.4				;  - ne: chyba, SDA je blokovno, nvrat z podprogramu
	BCF		STATUS, RP0		; banka 0
	RETLW	0

; Podprogram pro naten jedinho bitu pes I2C
; (vstup: bit DI)
BITIN
	BSF		STATUS, RP0		; banka 1
	BSF		TRISB, SDA		; SDA jako vstup
	BCF		STATUS, RP0		; banka 0
	BCF		buffer, DI		; buffer<DI> = 0
	BSF		STATUS, RP0		; banka 1
	BSF		TRISB, SCL		; SCL -> H
	BCF		STATUS, RP0		; banka 0
	BTFSC	PORTB, SCL		; SCL = L?
	GOTO	BIT1			;  - ne: skok na BIT1
; - ano: oeten chyby (SCL blokovn zazenm slave)
; TODO: zde by ml program vykat na uvolnn SCL obvodem slave 
	BSF		buffer, DI		;  - ano: buffer<DI> = 1
	GOTO	ACKOK
BIT1
	BTFSS	PORTB, SDA		; SDA = H?
	GOTO	ACKOK			;  - ne: skok na ACKOK
	BSF		buffer, DI		;  - ano: buffer<DI> = 1
ACKOK
	BSF		STATUS, RP0		; banka 1
	NOP
	BCF		TRISB, SCL		; SCL -> L
	BCF		STATUS, RP0		; banka 0
	RETLW	0

; Podprogram pro vysln jedinho bitu pes I2C
; (vstup: bit DO)
BITOUT
	BCF		STATUS, RP0		; banka 0
	BTFSS	buffer, DO		; buffer<DO> = 1?
	GOTO	BIT0			;  - ne: zpis 0, skok na BIT0
	BSF		STATUS, RP0		;  - ano: zpis 1, banka 1
	BSF		TRISB, SDA		; SDA = H (stav vysok impedance)
	BCF		STATUS, RP0		; banka 0
	BTFSC	PORTB, SDA		; SDA = L?
	GOTO	CLK1			;  - ne: skok na CLK1
; - ano: oeten chyby - na sbrnici patrn vysl dal zazen
; TODO: po zjitn tto chyby by ml program v reimu multimaster peruit vysln
	GOTO	CLK1
BIT0
	BSF		STATUS, RP0		; banka 1
	BCF		TRISB, SDA		; SDA = L
	NOP
	NOP
	NOP
CLK1
	BSF		STATUS, RP0		; banka 1
	BSF		TRISB, SCL		; SCL = H (stav vysok impedance)
	BCF		STATUS, RP0		; banka 0
	BTFSC	PORTB, SCL		; SCL = L?
	GOTO	BIT2			;  - ne: skok na BIT2
; - ano: oeten chyby (SCL blokovn zazenm slave)
; TODO: zde by ml program vykat na uvolnn SCL obvodem slave 

BIT2
	NOP
	NOP
	BSF		STATUS, RP0		; banka 1
	BCF		TRISB, SCL		; SCL -> L
	BCF		STATUS, RP0		; banka 0
	RETLW	0

	END