	title "Penos dat mezi dvma mikrokontrolry prostednictvm sbrnice I2C - obvod slave"

; Seril "Mikrokontrolry PIC", dl 56
; PE-AR 08/2012
;
; Vt pringl
;
; Popis: Data penesen pes I2C se zapisuj na port B (piny RB5 a RB7); s vyuitm peruen modulu SSP.

	LIST		p=16F88				; nastaven typu procesoru
	#INCLUDE	<P16F88.INC>		; definice promnnch pro tento procesor

; === NASTAVEN KONFIGURANHO SLOVA ===
	__CONFIG	_CONFIG1, _CP_OFF & _CCP1_RB0 & _DEBUG_OFF & _WRT_PROTECT_OFF & _CPD_OFF & _LVP_OFF & _BODEN_OFF & _MCLR_ON & _PWRTE_ON & _WDT_OFF & _INTRC_IO
	__CONFIG	_CONFIG2, _IESO_OFF & _FCMEN_OFF

; === DEFINICE PROMNNCH ===
W_temp		EQU	0x70	; doasn uloen obsahu registru W pi vyvoln peruen
status_temp	EQU	0x71	; doasn uloen obsahu registru STATUS pi vyvoln peruen

; === KONSTANTY ===
	#DEFINE	SLAVEADDR	b'10101110'		; adresa tohoto obvodu v reimu I2C slave

; === VEKTOR RESETU ===
RESET	ORG		0x0000		; zatek programu na adrese 0000h
		GOTO	INIT

; === OBSLUHA PERUEN ===
ISR		ORG		0x0004		; vektor peruen
; Uloen obsahu registr W a STATUS
	MOVWF	W_temp
	MOVF	STATUS,W		; W -> W_temp
	MOVWF	status_temp		; STATUS -> status_temp

	BCF		STATUS, RP0	; vbr banky 0
	MOVF	SSPBUF, W	; SSPBUF -> W
	BSF		STATUS, RP0	; vbr banky 1
	BTFSS	SSPSTAT, 5	; pijat bajt jsou data?
	GOTO	ISR_NODATA	;  - ne: pesko zpis na port B

	BCF		STATUS, RP0	; vbr banky 0
	ANDLW	b'11100000'	; maskovn bit
	MOVWF	PORTB		; W -> PORTB

ISR_NODATA
	BCF		STATUS, RP0	; vbr banky 0
	BCF		PIR1, SSPIF	; vynulovn pznaku peruen od modulu SSP
	
; Nahrn obsahu registr W a STATUS
	MOVF    status_temp, W
	MOVWF   STATUS
	SWAPF   W_temp, f
	SWAPF   W_temp, W
	RETFIE

INIT
; Inicializace port
	CLRF	PORTA		; inicializace portu A
	CLRF	PORTB		; inicializace portu B

	BSF		STATUS, RP0	; vbr banky 1

	CLRF	ANSEL		; piny AN<6:0> jako digitln I/O
	CLRF	TRISA		; piny RA<7:0> jako vstupy
	MOVLW	b'00010010'	
	MOVWF	TRISB		; piny RB4 (SCL) a RB1 (SDA) jako vstupy, ostatn jako vstupy

	MOVLW	b'00000000'	; vchoz nastaven stavovho registru modulu SSP
	MOVWF	SSPSTAT

	BSF		OSCCON, IRCF1	; Fosc = 250 kHz

	MOVLW	SLAVEADDR
	MOVWF	SSPADD		; nastaven adresy tohoto obvodu v reimu I2C slave

	MOVLW	b'00001000'
	MOVWF	PIE1		; povolen peruen od modulu SSP

	BCF		STATUS, RP0	; vbr banky 0

	MOVLW	b'00110110'
	MOVWF	SSPCON		; md slave se 7bitovou adresou, povolen SSP
	
	MOVLW	b'11000000'
	MOVWF	INTCON		; povolen perifernch peruen a celho systmu peruen
	
MAIN
	SLEEP
	NOP
	GOTO	MAIN

	END