	title "Oeten zkmit tlatek"

; Seril "Mikrokontrolry PIC", dl 58
; PE-AR 10/2012
;
; Vt pringl
;
; Popis: S kadm stiskem tlatka se posune svtc LED o jednu pozici. Program oetuje zkmity tlatek.

     LIST		p=16F88				; nastaven typu procesoru
     #INCLUDE	<P16F88.INC>		; definice promnnch pro tento procesor

; === NASTAVEN KONFIGURANHO SLOVA ===

	__CONFIG	_CONFIG1, _CP_OFF & _CCP1_RB0 & _DEBUG_OFF & _WRT_PROTECT_OFF & _CPD_OFF & _LVP_OFF & _BODEN_OFF & _MCLR_ON & _PWRTE_ON & _WDT_OFF & _INTRC_IO
	__CONFIG	_CONFIG2, _IESO_OFF & _FCMEN_OFF

; === DEFINICE PROMNNCH ===
	CBLOCK	0x20
		key_stat_pv		; pedchoz platn stav tlatek
		key_stat_p		; pedchoz stav tlatek
		key_stat_a		; aktuln stav tlatek
		porta_led		; stav LED pipojench k portu A
		read_cnt		; poet ten, kter jet zbv do uznn platnosti
		read_cnt0		; pomocn promnn pro intern cyklus tae read_cnt
		flags			; pznakov bity
	ENDC

; === DEFINICE KONSTANT ===
PB_KEY_MASK	EQU	b'10000000'	; definice pin, ke kterm jsou pipojena tlatka
READ_COUNT	EQU	.08			; poet ten stavu tlatek, po kter se nesm mnit stav,
							; aby byl povaovn za platn

; === DEFINICE BIT ===
KEY_CHNG	EQU	0x0			; zna, e byla zaregistrovna platn zmna stavu tlatek
RBPU		EQU	0x7

; === PROGRAM ===

RESET	ORG		0x0000		; zatek programu na adrese 0000h

; Inicializace portu
	CLRF	PORTA		; inicializace PORTA
	CLRF	PORTB		; inicializace PORTB

	BSF		STATUS, RP0	; vbr banky 1
	CLRF	ANSEL		; piny AN<6:0> jako digitln I/O
	CLRF	TRISA		; piny portu A jako vstupy
	MOVLW	b'10000000'
	MOVWF	TRISB		; piny RB<6:0> jako vstupy, RB7 jako vstup

	BCF		OPTION_REG, RBPU	; povolen pull-up rezistor na portu B
	
; Inicializace internho osciltoru
	MOVLW	b'01110000'
	MOVWF	OSCCON		; nastaven kmitotu internho osciltoru na 8 MHz
	
	BCF		STATUS, RP0	; vbr banky 0

; Inicializace promnnch
	MOVLW	b'10000000'
	MOVWF	key_stat_pv
	CLRF	flags
	MOVLW	b'00000001'
	MOVWF	porta_led	; rozsvi LED0
	CALL	WRITE_LED
	CALL	RESET_READ_COUNT

; Hlavn smyka programu
MAIN
	; zde me mikrokontrolr vykonvat dal innost
	CALL	SCAN_KEY
	BTFSS	flags, KEY_CHNG
	GOTO	MAIN
	BCF		flags, KEY_CHNG
	MOVF	key_stat_p, W
	MOVWF	key_stat_pv
	BTFSS	key_stat_p, 7
	CALL	CHANGE_LED
	GOTO	MAIN

; Podprogram pro naten stavu tlatka
SCAN_KEY
	MOVF	PORTB, W		; naten rovn na portu B
	ANDLW	PB_KEY_MASK		; zajmaj ns pouze piny, na kter jsou pipojena tlatka
	MOVWF	key_stat_a		; aktualizuj aktuln stav tlatek
	
	; Je aktuln stav tlatek stejn jako posledn platn stav?
	; Pokud ano, aktualizuj key_stat_p a vyresetuj ta. Pokud ne, nastala zmna a pokrauj na SCAN_KEY1.
	SUBWF	key_stat_pv, W
	BTFSS	STATUS, Z		; je key_stat_a = key_stat_pv?
	GOTO	SCAN_KEY1		; - ne: pokrauj na SCAN_KEY1
	MOVF	key_stat_a, W	; - ano: key_stat_p = key_stat_a
	MOVWF	key_stat_p
	CALL	RESET_READ_COUNT
	RETURN

SCAN_KEY1
	; Je aktuln stav tlatek stejn jako pedchoz?
	; Pokud ne, nastala zmna, aktualizuj key_stat_p a vyresetuj ta. Pokud ano, pokrauj na SCAN_KEY2.
	MOVF	key_stat_a, W
	SUBWF	key_stat_p, W
	BTFSC	STATUS, Z		; je key_stat_a = key_stat_p?
	GOTO	SCAN_KEY2		; - ano: pokrauj na SCAN_KEY2
	MOVF	key_stat_a, W	; - ne: key_stat_p = key_stat_a
	MOVWF	key_stat_p
	CALL	RESET_READ_COUNT
	RETURN	

SCAN_KEY2
	DECFSZ	read_cnt0, f	; read_cnt0 = read_cnt0 - 1; je vsledek = 0?
	RETURN					; - ne: nvrat
	DECFSZ	read_cnt, f		; - ano: read_cnt = read_cnt - 1; je vsledek = 0?
	RETURN					; - ne: nvrat
	BSF		flags, KEY_CHNG	; - ano: nastav pznakov bit pro zmnu stavu tl.
	CALL	RESET_READ_COUNT
	RETURN

; Podprogram pro rotaci LED
CHANGE_LED
	BCF		STATUS, C
	BTFSC	porta_led, 3	; svt-li posledn LED v ad, nastav C=1
	BSF		STATUS, C
	RLF		porta_led, f	; rotace LED
	CALL	WRITE_LED
	RETURN

; Podprogram pro zpis stavu LED na vstupy
WRITE_LED
	MOVLW	b'00001111'
	ANDWF	porta_led, W
	MOVWF	PORTA			; zpis na port A
	RETURN

RESET_READ_COUNT
	CLRF	read_cnt0
	MOVLW	READ_COUNT
	MOVWF	read_cnt
	RETURN

	END
