	title "Obsluha stisku tlatek s vyuitm asovae TMR0 a peruen"

; Seril "Mikrokontrolry PIC", dl 59
; PE-AR 11/2012
;
; Vt pringl
;
; Popis: Tlatky pipojenmi k pinm RB5 a RB7 lze mnit pozici svtc LED a jej jas.

     LIST		p=16F88				; nastaven typu procesoru
     #INCLUDE	<P16F88.INC>		; definice promnnch pro tento procesor

; === NASTAVEN KONFIGURANHO SLOVA ===
	__CONFIG	_CONFIG1, _CP_OFF & _CCP1_RB0 & _DEBUG_OFF & _WRT_PROTECT_OFF & _CPD_OFF & _LVP_OFF & _BODEN_OFF & _MCLR_ON & _PWRTE_ON & _WDT_OFF & _INTRC_IO
	__CONFIG	_CONFIG2, _IESO_OFF & _FCMEN_OFF

; === DEFINICE PROMNNCH ===
	CBLOCK	0x20
		key_stat_pv		; pedchoz platn stav tlatek
		key_stat_p		; pedchoz stav tlatek
		key_stat_a		; aktuln stav tlatek
		porta_led		; stav LED pipojench k portu A
		read_cnt		; poet ten, kter jet zbv do uznn platnosti
		flags			; pznakov bity
		led_br			; jas svtc LED
		led_br_tmp		; doasn promnn pro jas svtc LED
	ENDC
W_temp		EQU	0x70	; doasn uloen obsahu registru W pi vyvoln peruen
status_temp	EQU	0x71	; doasn uloen obsahu registru STATUS pi vyvoln peruen

; === DEFINICE KONSTANT ===
PB_KEY_MASK	EQU	b'11100000'	; definice pin portu B, ke kterm jsou pipojena tlatka
READ_COUNT	EQU	.08			; poet ten stavu tlatek, po kter se nesm mnit stav,
							; aby byl povaovn za platn

; === DEFINICE BIT ===
KEY_CHNG	EQU	0x0			; zna, e byla zaregistrovna platn zmna stavu tlatek

; === VEKTOR RESETU ===
RESET	ORG		0x0000		; zatek programu na adrese 0000h
		GOTO	INIT

; === OBSLUHA PERUEN ===
ISR		ORG		0x0004		; vektor peruen
; Uloen obsahu registr W a STATUS
	MOVWF	W_temp
	MOVF	STATUS,W		; W -> W_temp
	MOVWF	status_temp		; STATUS -> status_temp

	CALL	SCAN_KEY		; ten stavu tlatek
	BTFSS	flags, KEY_CHNG	; nastala platn zmna stavu tlatek?
	GOTO	ISR4			;  - ne: skok na ISR4 (konec obsluhy peruen)
	BCF		flags, KEY_CHNG	;  - ano: vynuluj pznak zmny stavu tlatek

	MOVLW	b'11100000'
	SUBWF	key_stat_pv, W	; pedchoz stav klves = dn stisknut klvesa?
	BTFSS	STATUS, Z
	GOTO	ISR2			;  - ne: skok na ISR2

	MOVLW	b'01100000'
	SUBWF	key_stat_a, W	; aktuln stisknut tlatko = TL2?
	BTFSC	STATUS, Z
	CALL	LED_ROT_RIGHT	;  - ano: rotace LED vpravo

	MOVLW	b'10100000'
	SUBWF	key_stat_a, W	; aktuln stisknut tlatko = TL1?
	BTFSC	STATUS, Z
	CALL	LED_ROT_LEFT	;  - ano: rotace LED vlevo

	GOTO	ISR3

ISR2
	MOVLW	b'11000000'
	SUBWF	key_stat_pv, W	; pedchoz stistknut tlatko = TL0?
	BTFSS	STATUS, Z
	GOTO	ISR3			;  - ne: skok na ISR3

	MOVLW	b'01000000'
	SUBWF	key_stat_a, W	; aktuln stisknut tlatko TL0+TL2?
	BTFSC	STATUS, Z
	CALL	LED_BR_INC		;  - ano: zvti jas LED

	MOVLW	b'10000000'
	SUBWF	key_stat_a, W	; aktuln stisknut tlatko TL0+TL1?
	BTFSC	STATUS, Z
	CALL	LED_BR_DEC		;  - ano: zmni jas LED

ISR3
	MOVF	key_stat_a, W	; pedchoz platn stav klves = aktuln
	MOVWF	key_stat_pv

ISR4
	BCF		INTCON, TMR0IF	; vynulovn pznaku peruen pi peteen asovae TMR0

; Nahrn obsahu registr W a STATUS
	MOVF    status_temp, W
	MOVWF   STATUS
	SWAPF   W_temp, f
	SWAPF   W_temp, W
	RETFIE

INIT
; Inicializace portu
	CLRF	PORTA		; inicializace PORTA
	CLRF	PORTB		; inicializace PORTB

	BSF		STATUS, RP0	; vbr banky 1
	CLRF	ANSEL		; piny AN<6:0> jako digitln I/O
	CLRF	TRISA		; piny portu A jako vstupy
	MOVLW	b'11100000'
	MOVWF	TRISB		; piny RB<4:0> jako vstupy, RB<7:5> jako vstup

	MOVLW	b'00000011'
	MOVWF	OPTION_REG	; povolen pull-up rezistor na portu B, povolen TMR0, peddlika 1:16
	
; Inicializace internho osciltoru
	MOVLW	b'01110000'
	MOVWF	OSCCON		; nastaven kmitotu internho osciltoru na 8 MHz
	
	BCF		STATUS, RP0	; vbr banky 0

	MOVLW	b'10100000'
	MOVWF	INTCON		; povolen peruen od asovae TMR0

; Inicializace promnnch
	MOVLW	b'11100000'
	MOVWF	key_stat_pv	; pedchoz platn stav tlatek = dn stisknut
	CLRF	flags
	MOVLW	b'00000001'
	MOVWF	porta_led	; rozsvi LED0
	MOVLW	0xFF
	MOVWF	led_br		; maximln jas LED
	CALL	RESET_READ_COUNT

; Hlavn smyka programu
MAIN
	MOVF	led_br, W	; led_br -> led_br_tmp
	MOVWF	led_br_tmp

	MOVLW	b'00001111'		
	ANDWF	porta_led, W
	MOVWF	PORTA		; zpis na port A

MAIN1
	DECFSZ	led_br_tmp, f	; led_br_tmp = led_br_tmp - 1, vsledek = 0?
	GOTO	MAIN1			;  - ne: skok na MAIN1

	MOVF	led_br, W		;  - ano: ...
	SUBLW	0x00
	MOVWF	led_br_tmp		; led_br_tmp = 256 - led_br
	CLRF	PORTA			; zhasni LED

MAIN2
	DECFSZ	led_br_tmp, f	; led_br_tmp = led_br_tmp - 1, vsledek = 0?
	GOTO	MAIN2			;  - ne: skok na MAIN2

	GOTO	MAIN			;  - ano: skok na MAIN

; Podprogram pro naten stavu tlatka
SCAN_KEY
	MOVF	PORTB, W		; naten rovn na portu B
	ANDLW	PB_KEY_MASK		; zajmaj ns pouze piny, na kter jsou pipojena tlatka
	MOVWF	key_stat_a		; aktualizuj aktuln stav tlatek
	
	; Je aktuln stav tlatek stejn jako posledn platn stav?
	; Pokud ano, aktualizuj key_stat_p a vyresetuj ta. Pokud ne, nastala zmna a pokrauj na SCAN_KEY1.
	SUBWF	key_stat_pv, W
	BTFSS	STATUS, Z		; je key_stat_a = key_stat_pv?
	GOTO	SCAN_KEY1		; - ne: pokrauj na SCAN_KEY1
	MOVF	key_stat_a, W	; - ano: key_stat_p = key_stat_a
	MOVWF	key_stat_p
	CALL	RESET_READ_COUNT
	RETURN

SCAN_KEY1
	; Je aktuln stav tlatek stejn jako pedchoz?
	; Pokud ne, nastala zmna, aktualizuj key_stat_p a vyresetuj ta. Pokud ano, pokrauj na SCAN_KEY2.
	MOVF	key_stat_a, W
	SUBWF	key_stat_p, W
	BTFSC	STATUS, Z		; je key_stat_a = key_stat_p?
	GOTO	SCAN_KEY2		; - ano: pokrauj na SCAN_KEY2
	MOVF	key_stat_a, W	; - ne: key_stat_p = key_stat_a
	MOVWF	key_stat_p
	CALL	RESET_READ_COUNT
	RETURN	

SCAN_KEY2
	DECFSZ	read_cnt, f		; read_cnt = read_cnt - 1; je vsledek = 0?
	RETURN					; - ne: nvrat
	BSF		flags, KEY_CHNG	; - ano: nastav pznakov bit pro zmnu stavu tl.
	CALL	RESET_READ_COUNT
	RETURN

; Podprogram pro rotaci LED vpravo
LED_ROT_RIGHT
	BTFSC	porta_led, 0	; svt-li LED0, nvrat
	RETURN
	BCF		STATUS, C
	RRF		porta_led, f
	RETURN

; Podprogram pro rotaci LED vlevo
LED_ROT_LEFT
	BTFSC	porta_led, 3	; svt-li LED3, nvrat
	RETURN
	BCF		STATUS, C
	RLF		porta_led, f
	RETURN

; Podprogram pro zvten jasu LED
LED_BR_INC
	BSF		STATUS, C
	RLF		led_br, f
	RETURN

; Podprogram pro zmenen jasu LED
LED_BR_DEC
	BTFSS	led_br, 1		; je-li led_br<1> = 0, nvrat (minimln jas)
	RETURN
	BCF		STATUS, C
	RRF		led_br, f
	RETURN

RESET_READ_COUNT
	MOVLW	READ_COUNT
	MOVWF	read_cnt
	RETURN

	END
