	title "Obsluha stisku tlatek s monost opakovn dan akce pi pidren tlatka"

; Seril "Mikrokontrolry PIC", dl 60
; PE-AR 12/2012
;
; Vt pringl
;
; Popis: Tlatky pipojenmi k pinm RB5 a RB7 lze mnit pozici svtc LED a jej jas.
;        Podrenm tlatka nebo kombinace tlatek se bude periodicky opakovat posledn akce.

     LIST		p=16F88				; nastaven typu procesoru
     #INCLUDE	<P16F88.INC>		; definice promnnch pro tento procesor

; === NASTAVEN KONFIGURANHO SLOVA ===
	__CONFIG	_CONFIG1, _CP_OFF & _CCP1_RB0 & _DEBUG_OFF & _WRT_PROTECT_OFF & _CPD_OFF & _LVP_OFF & _BODEN_OFF & _MCLR_ON & _PWRTE_ON & _WDT_OFF & _INTRC_IO
	__CONFIG	_CONFIG2, _IESO_OFF & _FCMEN_OFF

; === DEFINICE PROMNNCH ===
	CBLOCK	0x20
		key_stat_pv		; pedchoz platn stav tlatek
		key_stat_p		; pedchoz stav tlatek
		key_stat_a		; aktuln stav tlatek
		porta_led		; stav LED pipojench k portu A
		read_cnt		; poet ten, kter jet zbv do uznn platnosti
		flags			; pznakov bity
		led_br			; jas svtc LED
		led_br_tmp		; doasn promnn pro jas svtc LED
		cnt_long		; ta pro detekci dlouhho stisku tlatka
		cnt_rep_fast	; ta pro poet provedench akc pi dlouhm stisku, po kterm se opakovn zrychl
		key_stat_pa		; stav tlatek, pi kterm byla provedena posledn akce
	ENDC
W_temp		EQU	0x70	; doasn uloen obsahu registru W pi vyvoln peruen
status_temp	EQU	0x71	; doasn uloen obsahu registru STATUS pi vyvoln peruen

; === DEFINICE KONSTANT ===
PB_KEY_MASK	EQU	b'11100000'		; definice pin portu B, ke kterm jsou pipojena tlatka
READ_COUNT	EQU	.04				; poet ten stavu tlatek, po kter se nesm mnit stav,
								; aby byl povaovn za platn
NO_KEY_PRESSED	EQU	PB_KEY_MASK	; dn stisknut tlatko
TL0_PRESSED		EQU	b'11000000'	; stisknuto tlatko TL0
TL1_PRESSED		EQU	b'10100000'	; stisknuto tlatko TL1
TL2_PRESSED		EQU	b'01100000'	; stisknuto tlatko TL2
TL0_TL1_PRESSED	EQU	b'10000000'	; stisknuta tlatka TL0 a TL1
TL0_TL2_PRESSED	EQU	b'01000000'	; stisknuta tlatka TL0 a TL2

LONG_COUNT_INIT	EQU	.255	; poten prodleva pro opakovn akce pi dlouhm stisku
LONG_COUNT_NORM	EQU	.100	; normln prodleva pro opakovn akce pi dlouhm stisku
LONG_COUNT_FAST	EQU	.40		; krtk prodleva pro opakovn akce pi delm dren tlatka
REP_FAST_COUNT	EQU	.10		; poet opakovanch akc, po kterch se aktivuje rychl md opakovn

; === DEFINICE BIT ===
KEY_CHNG	EQU	0x0			; zna, e byla zaregistrovna platn zmna stavu tlatek

; === VEKTOR RESETU ===
RESET	ORG		0x0000		; zatek programu na adrese 0000h
		GOTO	INIT

; === OBSLUHA PERUEN ===
ISR		ORG		0x0004		; vektor peruen
; Uloen obsahu registr W a STATUS
	MOVWF	W_temp
	MOVF	STATUS,W		; W -> W_temp
	MOVWF	status_temp		; STATUS -> status_temp

	CALL	SCAN_KEY		; ten stavu tlatek
	BTFSS	flags, KEY_CHNG	; nastala platn zmna stavu tlatek?
	GOTO	ISR_LONG		;  - ne: skok na ISR_LONG (oeti dlouh stisk)
	BCF		flags, KEY_CHNG	;  - ano: vynuluj pznak zmny stavu tlatek

	MOVLW	0xFF
	MOVWF	key_stat_pa		; reset stavu tlatek pi posledn akci
	
	MOVLW	NO_KEY_PRESSED
	SUBWF	key_stat_pv, W	; pedchoz stav klves = dn stisknut klvesa?
	BTFSS	STATUS, Z
	GOTO	ISR2			;  - ne: skok na ISR2
	CALL	CHECK_KEY_LED_ROT_RIGHT
	CALL	CHECK_KEY_LED_ROT_LEFT
	GOTO	ISR_RESET_KEYS

ISR2
	MOVLW	TL0_PRESSED
	SUBWF	key_stat_pv, W	; pedchoz stisknut tlatko = TL0?
	BTFSS	STATUS, Z
	GOTO	ISR_RESET_KEYS	;  - ne: skok na ISR_RESET_KEYS
	CALL	CHECK_KEY_LED_BR_INC
	CALL	CHECK_KEY_LED_BR_DEC

ISR_RESET_KEYS
	MOVF	key_stat_a, W	; pedchoz platn stav klves = aktuln
	MOVWF	key_stat_pv
	
	MOVLW	LONG_COUNT_INIT
	MOVWF	cnt_long		; cnt_long = LONG_COUNT_INIT
	MOVLW	REP_FAST_COUNT
	MOVWF	cnt_rep_fast	; cnt_rep_fast = REP_FAST_COUNT
	
	GOTO	ISR_FINAL
	
; Detekce dlouhho stisku pro opakovn posledn akce
ISR_LONG
	MOVLW	NO_KEY_PRESSED
	SUBWF	key_stat_pv, W	; posledn platn stav tl. = dn stisknut tlatko?
	BTFSC	STATUS, Z
	GOTO	ISR_FINAL		;  - ano: skok na ISR_FINAL
	
	MOVF	key_stat_pa, W
	SUBWF	key_stat_pv, W	; posledn platn stav tl. = stav pi posledn proveden akci?
	BTFSS	STATUS, Z
	GOTO	ISR_FINAL		;  - ne: skok na ISR_FINAL
	
	DECFSZ	cnt_long, f		;  - ano: cnt_long = cnt_long - 1, vsledek = 0?
	GOTO	ISR_FINAL		;    - ne: skok na ISR_FINAL
							;    - ano: zkontroluj stav klves a prove pslun akce
	CALL	CHECK_KEY_LED_ROT_RIGHT
	CALL	CHECK_KEY_LED_ROT_LEFT
	CALL	CHECK_KEY_LED_BR_INC
	CALL	CHECK_KEY_LED_BR_DEC
	
	MOVF	cnt_rep_fast, f		; rychl md opakovn (cnt_rep_fast = 0)?
	BTFSC	STATUS, Z
	GOTO	ISR_LONG2			;  - ano: skok na ISR_LONG2
	MOVLW	LONG_COUNT_NORM		;  - ne: cnt_long = LONG_COUNT_NORM
	MOVWF	cnt_long
	
	DECFSZ	cnt_rep_fast, f		; cnt_rep_fast = cnt_rep_fast - 1, vsledek = 0?
	GOTO	ISR_FINAL			;  - ne: skok na ISR_FINAL

ISR_LONG2
	MOVLW	LONG_COUNT_FAST
	MOVWF	cnt_long			; cnt_long = LONG_COUNT_FAST

ISR_FINAL
	BCF		INTCON, TMR0IF	; vynulovn pznaku peruen pi peteen asovae TMR0

; Nahrn obsahu registr W a STATUS
	MOVF    status_temp, W
	MOVWF   STATUS
	SWAPF   W_temp, f
	SWAPF   W_temp, W
	RETFIE

; Podprogramy pro jednotliv akce v zvislosti na stisknutm tlatku
CHECK_KEY_LED_ROT_RIGHT
	MOVLW	TL2_PRESSED
	SUBWF	key_stat_a, W	; aktuln stisknut tlatko = TL2?
	BTFSS	STATUS, Z
	RETURN					;  - ne: nvrat
	CALL	LED_ROT_RIGHT	;  - ano: rotace LED vpravo
	GOTO	CHECK_KEY_LED_VALID
	
CHECK_KEY_LED_ROT_LEFT
	MOVLW	TL1_PRESSED
	SUBWF	key_stat_a, W	; aktuln stisknut tlatko = TL1?
	BTFSS	STATUS, Z
	RETURN					;  - ne: nvrat
	CALL	LED_ROT_LEFT	;  - ano: rotace LED vlevo
	GOTO	CHECK_KEY_LED_VALID
	
CHECK_KEY_LED_BR_INC
	MOVLW	TL0_TL2_PRESSED
	SUBWF	key_stat_a, W	; aktuln stisknut tlatko TL0+TL2?
	BTFSS	STATUS, Z
	RETURN					;  - ne: nvrat
	CALL	LED_BR_INC		;  - ano: zvti jas LED
	GOTO	CHECK_KEY_LED_VALID
	
CHECK_KEY_LED_BR_DEC
	MOVLW	TL0_TL1_PRESSED
	SUBWF	key_stat_a, W	; aktuln stisknut tlatko TL0+TL1?
	BTFSS	STATUS, Z
	RETURN					;  - ne: nvrat
	CALL	LED_BR_DEC		;  - ano: zmeni jas LED
	
CHECK_KEY_LED_VALID
	MOVF	key_stat_a, W	; zapi stav tlatek pro posledn platnou provedenou akci
	MOVWF	key_stat_pa
	RETURN
	
INIT
; Inicializace port
	CLRF	PORTA		; inicializace PORTA
	CLRF	PORTB		; inicializace PORTB

	BSF		STATUS, RP0	; vbr banky 1
	CLRF	ANSEL		; piny AN<6:0> jako digitln I/O
	CLRF	TRISA		; piny portu A jako vstupy
	MOVLW	b'11100000'
	MOVWF	TRISB		; piny RB<4:0> jako vstupy, RB<7:5> jako vstupy

	MOVLW	b'00000100'
	MOVWF	OPTION_REG	; povolen pull-up rezistor na portu B, povolen TMR0, peddlika 1:32
	
; Inicializace internho osciltoru
	MOVLW	b'01110000'
	MOVWF	OSCCON		; nastaven kmitotu internho osciltoru na 8 MHz
	
	BCF		STATUS, RP0	; vbr banky 0

	MOVLW	b'10100000'
	MOVWF	INTCON		; povolen peruen od asovae TMR0

; Inicializace promnnch
	MOVLW	NO_KEY_PRESSED
	MOVWF	key_stat_pv	; pedchoz platn stav tlatek = dn stisknut
	CLRF	flags
	MOVLW	b'00000001'
	MOVWF	porta_led	; rozsvi LED0
	MOVLW	0xFF
	MOVWF	led_br		; maximln jas LED
	CALL	RESET_READ_COUNT
	MOVLW	LONG_COUNT_INIT
	MOVWF	cnt_long
	MOVLW	REP_FAST_COUNT
	MOVWF	cnt_rep_fast

; Hlavn smyka programu
MAIN
	MOVF	led_br, W	; led_br -> led_br_tmp
	MOVWF	led_br_tmp

	MOVLW	b'00001111'		
	ANDWF	porta_led, W
	MOVWF	PORTA		; zpis na port A

MAIN1
	DECFSZ	led_br_tmp, f	; led_br_tmp = led_br_tmp - 1, vsledek = 0?
	GOTO	MAIN1			;  - ne: skok na MAIN1

	MOVF	led_br, W		;  - ano: ...
	SUBLW	0x00
	MOVWF	led_br_tmp		; led_br_tmp = 256 - led_br
	CLRF	PORTA			; zhasni LED

MAIN2
	DECFSZ	led_br_tmp, f	; led_br_tmp = led_br_tmp - 1, vsledek = 0?
	GOTO	MAIN2			;  - ne: skok na MAIN2

	GOTO	MAIN			;  - ano: skok na MAIN

; Podprogram pro naten stavu tlatka
SCAN_KEY
	MOVF	PORTB, W		; naten rovn na portu B
	ANDLW	PB_KEY_MASK		; zajmaj ns pouze piny, na kter jsou pipojena tlatka
	MOVWF	key_stat_a		; aktualizuj aktuln stav tlatek
	
	; Je aktuln stav tlatek stejn jako posledn platn stav?
	; Pokud ano, aktualizuj key_stat_p a vyresetuj ta. Pokud ne, nastala zmna a pokrauj na SCAN_KEY1.
	SUBWF	key_stat_pv, W
	BTFSS	STATUS, Z		; je key_stat_a = key_stat_pv?
	GOTO	SCAN_KEY1		; - ne: pokrauj na SCAN_KEY1
	MOVF	key_stat_a, W	; - ano: key_stat_p = key_stat_a
	MOVWF	key_stat_p
	CALL	RESET_READ_COUNT
	RETURN

SCAN_KEY1
	; Je aktuln stav tlatek stejn jako pedchoz?
	; Pokud ne, nastala zmna, aktualizuj key_stat_p a vyresetuj ta. Pokud ano, pokrauj na SCAN_KEY2.
	MOVF	key_stat_a, W
	SUBWF	key_stat_p, W
	BTFSC	STATUS, Z		; je key_stat_a = key_stat_p?
	GOTO	SCAN_KEY2		; - ano: pokrauj na SCAN_KEY2
	MOVF	key_stat_a, W	; - ne: key_stat_p = key_stat_a
	MOVWF	key_stat_p
	CALL	RESET_READ_COUNT
	RETURN	

SCAN_KEY2
	DECFSZ	read_cnt, f		; read_cnt = read_cnt - 1; je vsledek = 0?
	RETURN					; - ne: nvrat
	BSF		flags, KEY_CHNG	; - ano: nastav pznakov bit pro zmnu stavu tl.
	CALL	RESET_READ_COUNT
	RETURN

RESET_READ_COUNT
	MOVLW	READ_COUNT
	MOVWF	read_cnt
	RETURN

; Podprogram pro rotaci LED vpravo
LED_ROT_RIGHT
	BCF		porta_led, 4
	BTFSC	porta_led, 0	; svt-li LED0, nastav bit porta_led<4>
	BSF		porta_led, 4
	RRF		porta_led, f
	RETURN

; Podprogram pro rotaci LED vlevo
LED_ROT_LEFT
	BCF		STATUS, C
	BTFSC	porta_led, 3	; svt-li LED3, nastav bit C
	BSF		STATUS, C
	RLF		porta_led, f
	RETURN

; Podprogram pro zvten jasu LED
LED_BR_INC
	BSF		STATUS, C
	RLF		led_br, f
	RETURN

; Podprogram pro zmenen jasu LED
LED_BR_DEC
	BTFSS	led_br, 1		; je-li led_br<1> = 0, nvrat (minimln jas)
	RETURN
	BCF		STATUS, C
	RRF		led_br, f
	RETURN

	END
