	title "Obsluha krtkho, dlouhho a dvojitho stisku tlatka"

; Seril "Mikrokontrolry PIC", dl 61
; PE-AR 01/2013
;
; Vt pringl
;
; Popis: Program rozliuje ti typy stisku tlatka: normln, dlouh a dvojit. 
;        Pi normlnm stisku se zmn rove na pinu RA0, pi dlouhm stisku se 
;        zmn rove na pinu RA1 a pi dvojitm stisku se zmn rove na pinu RA2.

	LIST		p=16F88				; nastaven typu procesoru
	#INCLUDE	<P16F88.INC>		; definice promnnch pro tento procesor

; === NASTAVEN KONFIGURANHO SLOVA ===

	__CONFIG	_CONFIG1, _CP_OFF & _CCP1_RB0 & _DEBUG_OFF & _WRT_PROTECT_OFF & _CPD_OFF & _LVP_OFF & _BODEN_OFF & _MCLR_ON & _PWRTE_ON & _WDT_OFF & _INTRC_IO
	__CONFIG	_CONFIG2, _IESO_OFF & _FCMEN_OFF

; === DEFINICE PROMNNCH ===
	CBLOCK	0x20
		key_stat_pv		; pedchoz platn stav tlatek
		key_stat_a		; aktuln stav tlatek
		read_cnt		; poet ten, kter jet zbv do uznn platnosti
		flags			; pznakov bity
		keypress_type	; slou pro uloen typu stisku (krtk, dlouh, dvojit)
		key_off_cnt		; ta cykl, kdy je tlatko rozepnuto, pro odlien dvojitho
						; stisku od dvou jednoduchch
		key_on_cnt		; ta cykl, kdy je tlatko sepnuto, pro odlien krtkho
						; stisku od dlouhho
		delay_cnt		; pomocn promnn pro tn cykl v hlavn smyce
	ENDC

; === DEFINICE KONSTANT ===
PB_KEY_MASK	EQU	b'10000000'	; definice pin, ke kterm jsou pipojena tlatka
READ_COUNT	EQU	.08			; poet ten stavu tlatek, po kter se nesm mnit stav,
							; aby byl povaovn za platn
KEY_OFF_COUNT	EQU	.50		; poet cykl, kter reprezentuje dobu, po kterou je tlatko rozepnuto,
							; odliuje dvojit stisk od dvou po sob jdoucch krtkch stisk
KEY_ON_COUNT	EQU	.255	; poet cykl, kter reprezentuje dobu, po kterou je tlatko stisknuto,
							; odliuje krtk a dlouh stisk

; === DEFINICE BIT ===
KEY_CHNG	EQU	0x0			; zna, e byla zaregistrovna platn zmna stavu tlatek
RBPU		EQU	0x7			; pin pro povolen pull-up rezistor v reg. OPTION_REG
KEYPRESS_S	EQU	0x0			; jednoduch stisk, ze kterho se me nastat normln nebo dvojit stisk
KEYPRESS_L	EQU	0x2			; dlouh stisk tlatka
KEYPRESS_D	EQU	0x3			; dvojit stisk tlatka

; === PROGRAM ===
RESET	ORG		0x0000		; zatek programu na adrese 0000h

; Inicializace port
	CLRF	PORTA		; inicializace PORTA
	CLRF	PORTB		; inicializace PORTB

	BSF		STATUS, RP0	; vbr banky 1
	CLRF	ANSEL		; piny AN<6:0> jako digitln I/O
	CLRF	TRISA		; piny portu A jako vstupy
	MOVLW	b'10000000'
	MOVWF	TRISB		; piny RB<6:0> jako vstupy, RB7 jako vstup

	BCF		OPTION_REG, RBPU	; povolen pull-up rezistor na portu B
	
; Inicializace internho osciltoru
	MOVLW	b'01000000'
	MOVWF	OSCCON		; nastaven kmitotu internho osciltoru na 1 MHz
	
	BCF		STATUS, RP0	; vbr banky 0

; Inicializace promnnch
	MOVLW	b'10000000'
	MOVWF	key_stat_pv
	CLRF	flags
	CALL	RESET_READ_COUNT
	MOVLW	KEY_OFF_COUNT
	MOVWF	key_off_cnt
	MOVLW	KEY_ON_COUNT
	MOVWF	key_on_cnt
	CLRF	keypress_type

; Hlavn smyka programu
MAIN
	; zde me mikrokontrolr vykonvat dal innost

; Po kadch 256 cyklech hlavn smyky nati stav tlatka
	DECFSZ	delay_cnt, f	; delay_cnt - 1; je vsledek = 0?
	GOTO	MAIN			; - ne: skok na MAIN

	CALL	SCAN_KEY
	BTFSS	flags, KEY_CHNG	; nastala zmna stavu tlatka?
	GOTO	MAIN_NO_CHNG	;  - ne: skok na MAIN_NO_CHNG
	BCF		flags, KEY_CHNG	;  - ano: vynuluj bit KEY_CHNG
	MOVF	key_stat_a, W
	MOVWF	key_stat_pv		; pedchoz platn stav = aktuln stav

; Nastala zmna - tlatko bylo stisknuto nebo uvolnno	
	BTFSS	key_stat_a, 7	; je tlatko stisknuto?
	GOTO	MAIN_KEY_ON		;  - ano: skok na MAIN_KEY_ON
							;  - ne: rozhodni akci podle hodnoty promnn keypress_type
; Tlatko bylo prv uvolnno:
	MOVF	keypress_type, f
	BTFSC	STATUS, Z			; keypress_type = 0?
	GOTO	KEYPRESS_SINGLE		;  - ano: skok na KEYPRESS_SINGLE
	BTFSC	keypress_type, KEYPRESS_D	; nastaven pznak dvojitho stisku?
	GOTO	KEYPRESS_DOUBLE				;  - ano: skok na KEYPRESS_DOUBLE
	CLRF	keypress_type		; jin stav -> vynulovn keypress_type a skok na MAIN
	GOTO	MAIN

; Tlatko bylo prv stisknuto:
MAIN_KEY_ON
	MOVLW	KEY_ON_COUNT	; nastav vchoz hodnotu tae key_on_cnt
	MOVWF	key_on_cnt
	
	BTFSC	keypress_type, KEYPRESS_S	; nastaven pznak pedchozho stisku?
	BSF		keypress_type, KEYPRESS_D	;  - ano: nastav pznak dvojitho stisku
	GOTO	MAIN

; Nenastala zmna stavu tlatka
MAIN_NO_CHNG
	BTFSS	key_stat_pv, 7	; je tlatko stisknuto?
	GOTO	MAIN_NO_CHNG_ON	;  - ano: skok na MAIN_NO_CHNG_ON
							;  - ne: rozhodni akci podle hodnoty promnn keypress_type
; Tlatko nen stisknuto:
	BTFSS	keypress_type, KEYPRESS_S	; je nastaven bit KEYPRESS_S?
	GOTO	MAIN				;  - ne: skok na MAIN
	DECFSZ	key_off_cnt, f		;  - ano: key_off_cnt - 1, vsledek = 0?
	GOTO	MAIN				;   - ne: skok na MAIN
	GOTO	KEYPRESS_NORMAL		;   - ano: obsluha normlnho stisku

; Tlatko je stisknuto:
MAIN_NO_CHNG_ON
	MOVF	keypress_type, f
	BTFSS	STATUS, Z			; keypress_type = 0 (tj. vchoz stav)?
	GOTO	MAIN				;  - ne: skok na MAIN
	DECFSZ	key_on_cnt, f		;  - ano: key_on_cnt - 1, vsledek = 0?
	GOTO	MAIN				;    - ne: skok na MAIN
	CLRF	keypress_type		;    - ano: ...
	BSF		keypress_type, KEYPRESS_L	; ... nastaven pznaku dlouhho stisku
	GOTO	KEYPRESS_LONG				; obsluha dlouhho stisku

; Nastaven pznaku KEYPRESS_S
KEYPRESS_SINGLE
	BSF		keypress_type, KEYPRESS_S
	MOVLW	KEY_OFF_COUNT		; nastav vchoz hodnotu tae key_off_cnt
	MOVWF	key_off_cnt
	GOTO	MAIN

; Obsluha normlnho stisku
KEYPRESS_NORMAL
	CLRF	keypress_type
	MOVLW	b'00000001'
	XORWF	PORTA, f
	GOTO	MAIN
	
; Obsluha dlouhho stisku
KEYPRESS_LONG
	MOVLW	b'00000010'
	XORWF	PORTA, f
	GOTO	MAIN

; Obsluha dvojitho stisku
KEYPRESS_DOUBLE
	CLRF	keypress_type
	MOVLW	b'00000100'
	XORWF	PORTA, f
	GOTO	MAIN

; Podprogram pro naten stavu tlatka
SCAN_KEY
	MOVF	PORTB, W		; naten rovn na portu B
	ANDLW	PB_KEY_MASK		; zajmaj ns pouze piny, na kter jsou pipojena tlatka
	MOVWF	key_stat_a		; aktualizuj aktuln stav tlatek
	
	; Je aktuln stav tlatek stejn jako posledn platn stav?
	; Pokud ano, aktualizuj key_stat_p a vyresetuj ta. Pokud ne, nastala zmna a pokrauj na SCAN_KEY1.
	SUBWF	key_stat_pv, W
	BTFSS	STATUS, Z		; je key_stat_a = key_stat_pv?
	GOTO	SCAN_KEY1		; - ne: pokrauj na SCAN_KEY1
	CALL	RESET_READ_COUNT
	RETURN

SCAN_KEY1
	DECFSZ	read_cnt, f		; - ano: read_cnt = read_cnt - 1; je vsledek = 0?
	RETURN					; - ne: nvrat
	BSF		flags, KEY_CHNG	; - ano: nastav pznakov bit pro zmnu stavu tl.
	CALL	RESET_READ_COUNT
	RETURN

RESET_READ_COUNT
	MOVLW	READ_COUNT
	MOVWF	read_cnt
	RETURN

	END
